#define iChannel0 inputImageTexture
#define iChannel1 inputImageTexture2
#define texture(a, b) texture2D(a, fract(b))
#define fragColor gl_FragColor
precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 iResolution;
uniform float iTime;

const float PI = 3.141592653589793;

uniform vec2 canvasSize;
uniform vec2 targetPos;
uniform vec2 targetSize;
uniform float r;

vec2 platformUV(vec2 uv) {
    return vec2(uv.x, 1.- uv.y);
}

vec4 texture2Dpf(sampler2D samp, vec2 uv) {
    return texture2D(samp, platformUV(uv));
}

float rand(vec2 xy) {
    return fract(sin(dot(xy, vec2(12.9898, 78.233)))*43758.5453123);
}

bool inside(vec2 uv) {
    return (uv.x >= 0. && uv.x <= 1. && uv.y >= 0. && uv.y <= 1.);
}

vec4 directionalBlur(vec2 uv, sampler2D samp, float strength, vec2 speed) {
    float offset = rand(uv);
    vec4 col = vec4(0.0);
    float total = 0.;
    for (float t = 0.; t < 6.; t++) {
        float percent = (t + offset) / 6.0;
        float weight = (percent - percent * percent) * 0.6;
        vec2 uvoff = percent * strength * speed;
        vec2 uv1 = platformUV(uv + uvoff);
        vec2 uv2 = platformUV(uv - uvoff);
        vec4 color1 = texture2D(samp, uv1);
        vec4 color2 = texture2D(samp, uv2);
        if(!inside(uv1)){
            color1 *= 0.;
        }
        if(!inside(uv2)){
            color2 *= 0.;
        }
        col += color1*weight;
        col += color2*weight;
        total += weight;
    }
    return col/(2.*total);
}

vec2 move(vec2 uv, vec2 offset){
    return uv - offset;
}

float radian(float angle){
    return angle / 180.0 * PI;
}

vec2 rotate(vec2 uv, vec2 center, float angle){
    vec2 res = uv;
    float r = radian(angle);
    res = move(uv, center);
    res = vec2(res.x * cos(r) + res.y * sin(r), res.y * cos(r) - res.x * sin(r));
    res = move(res, -center);
    return res;
}

void main(){
    vec2 fragCoord = textureCoordinate * canvasSize;
    vec2 ts = targetSize;
    vec2 tpos = fragCoord;
    tpos = move(tpos, targetPos);
    tpos = rotate(tpos, ts / 2.0, r);

    vec2 uv = tpos / ts;

    uv = platformUV(uv);
    vec4 color = vec4(1.);

    float fractTime = fract(iTime*.5);
    if (fractTime > .4) {
        uv = platformUV(uv);
        if (!inside(uv)){
            discard;
        }
        color = texture2D(inputImageTexture, uv);
    } else {
        fractTime /= .4;
        vec2 direction = vec2(-1., 1.);
        float strength = .2*abs(sin(fractTime*PI)) * 0.3; // 2022年12月28日 20点51分 为了减少效果乘了0.3
        color = directionalBlur(uv, inputImageTexture, strength, direction);
    }

    gl_FragColor = color;
}
